@@ -29,6 +29,7 @@
 #include <QProcess>
 #include <QDirIterator>
 #include <QTextStream>
+#include <QX11Info>
 
 #include <KCrash>
 #include <KLocalizedString>
@@ -37,8 +38,11 @@
 #include <KStartupInfo>
 #include <KWindowSystem>
 #include <kwindowsystem_version.h>
+#include <KX11Extras>
+#include <KMessageBox>
 
 #include <iostream>
+#include <unistd.h>
 
 KonquerorApplication::KonquerorApplication(int &argc, char **argv)
     : QApplication(argc, argv)
@@ -56,19 +60,39 @@ KonquerorApplication::KonquerorApplication(int &argc, char **argv)
                  SLOT(slotRemoveFromCombo(QString,QDBusMessage)));
     dbus.connect(QString(), KONQ_MAIN_PATH, dbusInterface, QStringLiteral("comboCleared"), this, SLOT(slotComboCleared(QDBusMessage)));
 
-#ifdef WEBENGINEPART_DICTIONARY_DIR
+#ifdef WEBENGINEPART_OWN_DICTIONARY_DIR
+    //If the user alredy set QTWEBENGINE_DICTIONARIES_PATH, don't override it
     if (!qEnvironmentVariableIsSet("QTWEBENGINE_DICTIONARIES_PATH")) {
-        qputenv("QTWEBENGINE_DICTIONARIES_PATH", WEBENGINEPART_DICTIONARY_DIR);
+        qputenv("QTWEBENGINE_DICTIONARIES_PATH", WEBENGINEPART_OWN_DICTIONARY_DIR);
     }
 #endif
 
+    m_runningAsRootBehavior = checkRootBehavior();
+
     QByteArray flags = qgetenv("QTWEBENGINE_CHROMIUM_FLAGS");
     flags.append(" --enable-features=WebRTCPipeWireCapturer");
-    qputenv("QTWEBENGINE_CHROMIUM_FLAGS", flags);
+    if (m_runningAsRootBehavior == RunInDangerousMode) {
+        flags.append (" --no-sandbox");
+    }
 
+    qputenv("QTWEBENGINE_CHROMIUM_FLAGS", flags);
     KLocalizedString::setApplicationDomain("konqueror");
 }
 
+KonquerorApplication::KonquerorAsRootBehavior KonquerorApplication::checkRootBehavior()
+{
+    uid_t uid = geteuid();
+    if (uid == 0) {
+        QString msg = i18n("<p>You're running Konqueror as root. This requires enabling a highly insecure mode in the browser component.</p><p>What do you want to do?</p>");
+        KGuiItem enableInsecureMode(QLatin1String("Enable the insecure mode")); //Continue
+        KGuiItem exitKonq(QLatin1String("Exit Konqueror")); //Cancel
+        KMessageBox::ButtonCode ans = KMessageBox::warningContinueCancel(nullptr, msg, QString(), enableInsecureMode, exitKonq);
+        return ans == KMessageBox::Continue ? RunInDangerousMode : PreventRunningAsRoot;
+    } else {
+        return NotRoot;
+    }
+}
+
 void KonquerorApplication::slotReparseConfiguration()
 {
     KSharedConfig::openConfig()->reparseConfiguration();
@@ -80,6 +104,7 @@ void KonquerorApplication::slotReparseConfiguration()
             window->reparseConfiguration();
         }
     }
+    emit configurationChanged();
 }
 
 void KonquerorApplication::slotAddToCombo(const QString &url, const QDBusMessage &msg)
@@ -103,7 +128,7 @@ void KonquerorApplication::setupAboutData()
     m_aboutData.setShortDescription(i18n("Web browser, file manager and document viewer."));
     m_aboutData.addLicense(KAboutLicense::GPL_V2);
     m_aboutData.setCopyrightStatement(i18n("(C) 1999-2016, The Konqueror developers"));
-    m_aboutData.setHomepage("https://konqueror.org/");
+    m_aboutData.setHomepage("https://apps.kde.org/konqueror");
 
     m_aboutData.addAuthor(i18n("Stefano Crocco"), i18n("Current maintainer"), "stefano.crocco@alice.it");
     m_aboutData.addAuthor(i18n("David Faure"), i18n("Developer (framework, parts, JavaScript, I/O library) and former maintainer"), "faure@kde.org");
@@ -191,7 +216,8 @@ int KonquerorApplication::startFirstInstance()
 
     const int ret = exec();
 
-    bool alwaysPreload = KonqSettings::alwaysHavePreloaded();
+    //Don't preload if Konqueror is run as root
+    bool alwaysPreload = m_runningAsRootBehavior == NotRoot && KonqSettings::alwaysHavePreloaded();
 
     // Delete all KonqMainWindows, so that we don't have
     // any parts loaded when KLibLoader::cleanUp is called.
@@ -216,6 +242,10 @@ int KonquerorApplication::startFirstInstance()
 
 int KonquerorApplication::start()
 {
+    if (m_runningAsRootBehavior == PreventRunningAsRoot) {
+        return 0;
+    }
+
     setupAboutData();
     setupParser();
 
@@ -224,12 +254,20 @@ int KonquerorApplication::start()
     m_parser.process(*this);
     m_aboutData.processCommandLine(&m_parser);
 
-    KDBusService dbusService(KDBusService::Unique);
-    auto activateApp = [this](const QStringList &arguments, const QString &workingDirectory) {
-        m_parser.parse(arguments);
-        performStart(workingDirectory, false);
-    };
-    QObject::connect(&dbusService, &KDBusService::activateRequested, activateApp);
+    //Explicitly disable reusing existing instances if running as root. The behavior is not clear
+    //and could be dangerous if new windows are unknowingly launched as root.
+    //I thought that creating the KDBusService instance inside the if block should do it; however, doing
+    //so prevents the new window to be created because the new instance produces a critical failure with message:
+    //Couldn't register name 'org.kde.konqueror' with DBUS - another process owns it already!
+    //Moving the service creation outside the if block seems to solve the issue.
+    KDBusService dbusService(m_runningAsRootBehavior == NotRoot ? KDBusService::Unique : KDBusService::Multiple | KDBusService::NoExitOnFailure);
+    if (m_runningAsRootBehavior == NotRoot) {
+        auto activateApp = [this](const QStringList &arguments, const QString &workingDirectory) {
+            m_parser.parse(arguments);
+            performStart(workingDirectory, false);
+        };
+        QObject::connect(&dbusService, &KDBusService::activateRequested, activateApp);
+    }
 
     return startFirstInstance();
 }
@@ -242,7 +280,19 @@ int KonquerorApplication::performStart(const QString& workingDirectory, bool fir
         listSessions();
         return 0;
     } else if (m_parser.isSet("open-session")) {
-        return openSession(m_parser.value("open-session"));
+        //If the given session can't be opened for any reason, inform the user
+        QString sessionName = m_parser.value("open-session");
+        int result = openSession(sessionName);
+        if (result != 0) {
+            KMessageBox::sorry(nullptr, i18nc("The session asked by the user doesn't exist or can't be opened",
+                                              "Session %1 couldn't be opened", sessionName));
+        }
+        //If there was an error opening the session and this is the first instance, don't return immediately
+        //as this would lead to the application being run but with no windows open. Instead, open an empty
+        //window
+        if (result != 0 && firstInstance) {
+            return result;
+        }
     }
 
     //We check for the --preload switch before attempting recovering session because we shouldn't
@@ -252,7 +302,8 @@ int KonquerorApplication::performStart(const QString& workingDirectory, bool fir
         return 0;
     }
 
-    if (!m_sessionRecoveryAttempted) {
+    //Don't attempt to restore session when running as root
+    if (!m_sessionRecoveryAttempted && m_runningAsRootBehavior == NotRoot) {
         // Ask the user to recover session if applicable
         KonqSessionManager::self()->askUserToRestoreAutosavedAbandonedSessions();
         m_sessionRecoveryAttempted = true;
@@ -267,13 +318,9 @@ int KonquerorApplication::performStart(const QString& workingDirectory, bool fir
 
     KonqMainWindow *mw = result.first;
     if (!firstInstance && mw) {
-#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5,62,0)
         mw ->setAttribute(Qt::WA_NativeWindow, true);
-        KStartupInfo::setNewStartupId(mw->windowHandle(), KStartupInfo::startupId());
-#else
-        KStartupInfo::setNewStartupId(mw, KStartupInfo::startupId());
-#endif
-        KWindowSystem::forceActiveWindow(mw->winId());
+        KStartupInfo::setNewStartupId(mw->windowHandle(), QX11Info::nextStartupId());
+        KX11Extras::forceActiveWindow(mw->winId());
     }
 
     return result.second;
