@@ -1,21 +1,8 @@
 /* This file is part of the KDE project
-   Copyright 2007 David Faure <faure@kde.org>
-   Copyright 2007 Eduardo Robles Elvira <edulix@gmail.com>
+    SPDX-FileCopyrightText: 2007 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2007 Eduardo Robles Elvira <edulix@gmail.com>
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "konqundomanager.h"
@@ -88,7 +75,7 @@ bool KonqUndoManager::undoAvailable() const
     if (!m_closedItemList.isEmpty() || m_cwManager->undoAvailable()) {
         return true;
     } else {
-        return (m_supportsFileUndo && KIO::FileUndoManager::self()->undoAvailable());
+        return (m_supportsFileUndo && KIO::FileUndoManager::self()->isUndoAvailable());
     }
 }
 
@@ -96,7 +83,7 @@ QString KonqUndoManager::undoText() const
 {
     if (!m_closedItemList.isEmpty()) {
         const KonqClosedItem *closedItem = m_closedItemList.first();
-        if (!m_supportsFileUndo || !KIO::FileUndoManager::self()->undoAvailable() || closedItem->serialNumber() > KIO::FileUndoManager::self()->currentCommandSerialNumber()) {
+        if (!m_supportsFileUndo || !KIO::FileUndoManager::self()->isUndoAvailable() || closedItem->serialNumber() > KIO::FileUndoManager::self()->currentCommandSerialNumber()) {
             const KonqClosedTabItem *closedTabItem =
                 dynamic_cast<const KonqClosedTabItem *>(closedItem);
             if (closedTabItem) {
@@ -108,7 +95,7 @@ QString KonqUndoManager::undoText() const
             return KIO::FileUndoManager::self()->undoText();
         }
 
-    } else if (m_supportsFileUndo && KIO::FileUndoManager::self()->undoAvailable()) {
+    } else if (m_supportsFileUndo && KIO::FileUndoManager::self()->isUndoAvailable()) {
         return KIO::FileUndoManager::self()->undoText();
     }
 
@@ -127,7 +114,7 @@ void KonqUndoManager::undo()
         KonqClosedItem *closedItem = m_closedItemList.first();
 
         // Check what to undo
-        if (!m_supportsFileUndo || !fileUndoManager->undoAvailable() || closedItem->serialNumber() > fileUndoManager->currentCommandSerialNumber()) {
+        if (!m_supportsFileUndo || !fileUndoManager->isUndoAvailable() || closedItem->serialNumber() > fileUndoManager->currentCommandSerialNumber()) {
             undoClosedItem(0);
             return;
         }
