@@ -34,6 +34,7 @@
 #include <QMimeDatabase>
 #include <QWebEngineProfile>
 #include <QFileDialog>
+#include <QFileInfo>
 
 bool UrlLoader::embedWithoutAskingToSave(const QString &mimeType)
 {
@@ -100,6 +101,9 @@ void UrlLoader::start()
         detectSettingsForRemoteFiles();
     }
 
+    if (hasError()) {
+        m_mimeType = QStringLiteral("text/html");
+    }
     if (isMimeTypeKnown(m_mimeType)) {
         KService::Ptr preferredService = KApplicationTrader::preferredService(m_mimeType);
         if (serviceIsKonqueror(preferredService)) {
@@ -117,6 +121,10 @@ bool UrlLoader::isViewLocked() const
 
 void UrlLoader::decideAction()
 {
+    if (hasError()) {
+        m_action = OpenUrlAction::Embed;
+        return;
+    }
     m_action = decideExecute();
     switch (m_action) {
         case OpenUrlAction::Execute:
@@ -264,25 +272,30 @@ UrlLoader::OpenUrlAction UrlLoader::decideExecute() const {
     bool canDisplay = !KParts::PartLoader::partsForMimeType(m_mimeType).isEmpty();
 
     KMessageBox::ButtonCode code;
-    KGuiItem executeGuiItem(i18nc("Execute an executable file", "Execute it"));
+    KGuiItem executeGuiItem(i18nc("Execute an executable file", "Execute"),
+                            QIcon::fromTheme(QStringLiteral("system-run")));
+    KGuiItem displayGuiItem(i18nc("Display an executable file", "Display"),
+                            QIcon::fromTheme(QStringLiteral("document-preview")));
     QString dontShowAgainId(QLatin1String("AskExecuting")+m_mimeType);
 
     if (canDisplay) {
-        code = KMessageBox::questionYesNoCancel(m_mainWindow, i18nc("The user has to decide whether to execute an executable file or display it",
-                                                                                        "<tt>%1</tt> can be executed. Do you want to execute it or to display it?", m_url.path()),
-                                                                    QString(), executeGuiItem, KGuiItem(i18nc("Display an executable file", "Display it")),
-                                                                    KStandardGuiItem::cancel(), dontShowAgainId, KMessageBox::Dangerous);
+        code = KMessageBox::questionTwoActionsCancel(m_mainWindow,
+                                                     xi18nc("@info The user has to decide whether to execute an executable file or display it",
+                                                            "<filename>%1</filename> can be executed. Do you want to execute it or to display it?", m_url.path()),
+                                                     QString(), executeGuiItem, displayGuiItem,
+                                                     KStandardGuiItem::cancel(), dontShowAgainId, KMessageBox::Dangerous);
     } else {
-        code = KMessageBox::questionYesNo(m_mainWindow, i18nc("The user has to decide whether to execute an executable file or not",
-                                                                                        "<tt>%1</tt> can be executed. Do you want to execute it?", m_url.path()),
-                                                                    QString(), executeGuiItem, KStandardGuiItem::cancel(),
-                                                                    dontShowAgainId, KMessageBox::Dangerous);}
+        code = KMessageBox::questionTwoActions(m_mainWindow,
+                                               xi18nc("@info The user has to decide whether to execute an executable file or not",
+                                                      "<filename>%1</filename> can be executed. Do you want to execute it?", m_url.path()),
+                                               QString(), executeGuiItem, KStandardGuiItem::cancel(),
+                                               dontShowAgainId, KMessageBox::Dangerous);}
     switch (code) {
-        case KMessageBox::Yes:
+        case KMessageBox::PrimaryAction:
             return OpenUrlAction::Execute;
         case KMessageBox::Cancel:
             return OpenUrlAction::DoNothing;
-        case KMessageBox::No:
+        case KMessageBox::SecondaryAction:
             //The "No" button actually corresponds to the "Cancel" action if the file can't be displayed
             return canDisplay ? OpenUrlAction::UnknwonAction : OpenUrlAction::DoNothing;
         default: //This is here only to avoid a compiler warning
@@ -378,12 +391,32 @@ void UrlLoader::detectSettingsForRemoteFiles()
     }
 }
 
+int UrlLoader::checkAccessToLocalFile(const QString& path)
+{
+    QFileInfo info(path);
+    bool fileExists = info.exists();
+    if (!info.isReadable()) {
+        QFileInfo parentInfo(info.dir().path());
+        if (parentInfo.isExecutable() && !fileExists) {
+            return KIO::ERR_DOES_NOT_EXIST;
+        } else {
+            return KIO::ERR_CANNOT_OPEN_FOR_READING;
+        }
+    } else if (info.isDir() && !info.isExecutable()) {
+        return KIO::ERR_CANNOT_ENTER_DIRECTORY;
+    } else {
+        return 0;
+    }
+}
+
 void UrlLoader::detectSettingsForLocalFiles()
 {
     if (!m_url.isLocalFile()) {
         return;
     }
 
+    m_jobErrorCode = checkAccessToLocalFile(m_url.path());
+
     if (!m_mimeType.isEmpty()) {
         // Generic mechanism for redirecting to tar:/<path>/ when clicking on a tar file,
         // zip:/<path>/ when clicking on a zip file, etc.
@@ -438,6 +471,7 @@ bool UrlLoader::shouldEmbedThis() const
 void UrlLoader::embed()
 {
     if (m_jobErrorCode) {
+        QUrl url = m_url;
         m_url = KParts::BrowserRun::makeErrorUrl(m_jobErrorCode, m_url.scheme(), m_url);
         m_mimeType = QStringLiteral("text/html");
         m_part = findPartById(QStringLiteral("webenginepart"));
