@@ -17,6 +17,7 @@
 #include "webenginewallet.h"
 #include <webenginepart_debug.h>
 #include "webenginepartcontrols.h"
+#include "navigationrecorder.h"
 
 #include <QWebEngineCertificateError>
 #include <QWebEngineSettings>
@@ -33,7 +34,6 @@
 #include <KIO/AuthInfo>
 #include <KIO/Job>
 #include <KIO/AccessManager>
-#include <KIO/Scheduler>
 #include <KIO/CommandLauncherJob>
 #include <KParts/HtmlExtension>
 #include <KUserTimestamp>
@@ -51,6 +51,7 @@
 #include <QAuthenticator>
 #include <QApplication>
 #include <QNetworkReply>
+#include <QTimer>
 #include <QWebEngineHistory>
 #include <QWebEngineHistoryItem>
 #include <QWebEngineDownloadItem>
@@ -66,6 +67,9 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
          m_ignoreError(false),
          m_part(part),
          m_passwdServerClient(new KPasswdServerClient)
+#ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+         , m_dropOperationTimer(new QTimer(this))
+#endif
 {
     if (view())
         WebEngineSettings::self()->computeFontSizes(view()->logicalDpiY());
@@ -84,6 +88,11 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
             this, &WebEnginePage::slotAuthenticationRequired);
     connect(this, &QWebEnginePage::fullScreenRequested, this, &WebEnginePage::changeFullScreenMode);
 
+#ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+    connect(this, &QWebEnginePage::loadStarted, this, [this](){m_dropOperationTimer->stop();});
+    m_dropOperationTimer->setSingleShot(true);
+#endif
+
     //If this part is displaying the developer tools for another part, inform the other page it's not displaying the developer tools anymore.
     //I'm not sure this is needed, but I think it's better to do it, just to be on the safe side
     auto unsetInspectedPageIfNeeded = [this](bool ok) {
@@ -93,6 +102,7 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
     };
     connect(this, &QWebEnginePage::loadFinished, this, unsetInspectedPageIfNeeded);
 
+    WebEnginePartControls::self()->navigationRecorder()->registerPage(this);
     m_part->downloadManager()->addPage(this);
 }
 
@@ -132,7 +142,7 @@ static void checkForDownloadManager(QWidget* widget, QString& cmd)
     cmd = exeName;
 }
 
-void WebEnginePage::download(QWebEngineDownloadItem *it, bool newWindow)
+void WebEnginePage::downloadItem(QWebEngineDownloadItem *it, bool newWindow)
 {
     QUrl url = it->url();
     // Integration with a download manager...
@@ -154,18 +164,34 @@ void WebEnginePage::download(QWebEngineDownloadItem *it, bool newWindow)
     askBrowserToOpenUrl(url, it->mimeType(), urlArgs, bArgs);
 }
 
-void WebEnginePage::requestOpenFileAsTemporary(const QUrl& url, const QString &mimeType, bool newWindow)
+void WebEnginePage::requestOpenFileAsTemporary(const QUrl& url, const QString &mimeType, bool newWindow, bool newTab)
 {
     KParts::BrowserArguments bArgs;
     bArgs.setForcesNewWindow(newWindow);
+    bArgs.setNewTab(newTab);
     KParts::OpenUrlArguments oArgs;
     oArgs.setMimeType(mimeType);
     oArgs.metaData().insert("konq-temp-file", "1");
     emit part()->browserExtension()->openUrlRequest(url, oArgs, bArgs);
 }
 
+# ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+void WebEnginePage::setDropOperationStarted()
+{
+    m_dropOperationTimer->start(100);
+}
+#endif
+
+
 QWebEnginePage *WebEnginePage::createWindow(WebWindowType type)
 {
+#ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+    if (m_dropOperationTimer->isActive()) {
+        m_dropOperationTimer->stop();
+        return this;
+    }
+#endif
+
     //qCDebug(WEBENGINEPART_LOG) << "window type:" << type;
     // Crete an instance of NewWindowPage class to capture all the
     // information we need to create a new window. See documentation of
@@ -312,6 +338,9 @@ bool WebEnginePage::acceptNavigationRequest(const QUrl& url, NavigationType type
     // Honor the enabling/disabling of plugins per host.
     settings()->setAttribute(QWebEngineSettings::PluginsEnabled, WebEngineSettings::self()->isPluginsEnabled(reqUrl.host()));
 
+    if (isMainFrame) {
+        emit mainFrameNavigationRequested(this, url);
+    }
     return QWebEnginePage::acceptNavigationRequest(url, type, isMainFrame);
 }
 
@@ -842,10 +871,10 @@ bool NewWindowPage::acceptNavigationRequest(const QUrl &url, NavigationType type
                                           i18n("<qt>This site is requesting to open a popup window to"
                                                "<p>%1</p><br/>Do you want to allow this?</qt>",
                                                KStringHandler::rsqueeze(reqUrl.toDisplayString().toHtmlEscaped(), 100)));
-                if (KMessageBox::questionYesNo(view(), message,
+                if (KMessageBox::questionTwoActions(view(), message,
                                                i18n("Javascript Popup Confirmation"),
                                                KGuiItem(i18n("Allow")),
-                                               KGuiItem(i18n("Do Not Allow"))) == KMessageBox::No) {
+                                               KGuiItem(i18n("Do Not Allow"))) == KMessageBox::SecondaryAction) {
                     // TODO: Implement support for dealing with blocked pop up windows.
                     this->deleteLater();
                     return false;
@@ -895,7 +924,7 @@ bool NewWindowPage::acceptNavigationRequest(const QUrl &url, NavigationType type
         //Set the create new window flag to false...
         m_createNewWindow = false;
         if (webenginePart) {
-            QTimer::singleShot(0, [webenginePart, url](){emit webenginePart->browserExtension()->openUrlRequest(url);});
+            QTimer::singleShot(0, webenginePart, [webenginePart, url](){emit webenginePart->browserExtension()->openUrlRequest(url);});
             return false;
         }
 
