@@ -12,6 +12,7 @@
 #define WEBENGINEPAGE_H
 
 #include "websslinfo.h"
+#include "kwebenginepartlib_export.h"
 
 #include <KParts/BrowserExtension>
 #include <QWebEnginePage>
@@ -49,9 +50,9 @@ public:
      */
     void setSslInfo (const WebSslInfo &other);
 
-    void download(QWebEngineDownloadItem *it, bool newWindow = false);
+    void downloadItem(QWebEngineDownloadItem *it, bool newWindow = false);
 
-    void requestOpenFileAsTemporary(const QUrl &url, const QString &mimeType = "", bool newWindow = false);
+    void requestOpenFileAsTemporary(const QUrl &url, const QString &mimeType = "", bool newWindow = false, bool newTab = false);
 
     void setStatusBarText(const QString &text);
 
@@ -72,6 +73,26 @@ public:
      */
     void setPart(WebEnginePart *part);
 
+#ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+    /**
+     * @brief Informs the page that a drop operation has been started
+     *
+     * Calling this method changes the behavior of createWindow()
+     * @note Since `QWebEngineView` doesn't provide a way to tell when a drop operation actually end,
+     * there's no function to inform that the drop operation has ended. The page considers the
+     * drop operation to have ended when one the following happens:
+     * - createWindow() is called
+     * - the `loadStarted` signal is emitted
+     * - 100ms have elapsed since the last call to this method (this time interval has been chosen arbitrary: it should
+     *   be enough for the drop operation to have actually ended but short enough to make it unlikely that
+     *   the user has started another action)
+     * @see m_dropOperationTimer
+     * @see createWindow()
+     * @see WebEngineView::dropEvent
+     */
+    void setDropOperationStarted();
+#endif
+
 Q_SIGNALS:
     /**
      * This signal is emitted whenever a user cancels/aborts a load resource
@@ -79,7 +100,7 @@ Q_SIGNALS:
      */
     void loadAborted(const QUrl &url);
 
-    void leavingPage(QWebEnginePage::NavigationType type);
+    void mainFrameNavigationRequested(WebEnginePage *page, const QUrl);
 
 protected:
     /**
@@ -89,10 +110,19 @@ protected:
     WebEnginePart* part() const;
 
     /**
+     * @brief Override of `QWebEnginePage::createWindow`
+     *
      * Reimplemented for internal reasons, the API is not affected.
-     * @internal
+     *
+     * By default, a new NewWindowPage will be returned; however, calling setDropOperationStarted changes
+     * this behavior: in this case, no pages will be created and the function returns `this`. The default
+     * behavior is restored when the drop operation ends (see setDropOperationStarted()).
+     * @param type the window type to create. This is ignored if setDropOperationStarted has been called with `true`
+     * @return a new NewWindowPage or `this` if within a drop operation
+     * @see setDropOperationStarted
      */
     QWebEnginePage* createWindow(WebWindowType type) override;
+protected:
 
     /**
      * Reimplemented for internal reasons, the API is not affected.
@@ -172,6 +202,16 @@ private:
     QPointer<WebEnginePart> m_part;
 
     QScopedPointer<KPasswdServerClient> m_passwdServerClient;
+
+#ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
+    /**
+     * @brief Timer used to decide whether a drop operation is happening
+     *
+     * A drop operation is happening if this timer is active
+     * @see setDropOperationStarted for more details
+     */
+    QTimer *m_dropOperationTimer;
+#endif
 };
 
 
